package legion.muyue.best2048;

import java.util.ArrayList;
import java.util.List; // Import explicite
import java.util.Random;
import android.util.Log;


public class Game {

    private int[][] gameBoard;
    private Random random;
    private int score = 0;
    private int highScore = 0; // Ajout variable high score


    public Game() {
        gameBoard = new int[4][4];
        random = new Random();
        loadHighScore(); // Charger au démarrage
    }

    public int getGameBoard(int x, int y) {
        return gameBoard[x][y];
    }

    public int getScore() {
        return score;
    }

    public int getHighScore() {
        return highScore;
    }

    // Charger le meilleur score (stub)
    private void loadHighScore() {
        //  highScore = sharedPreferences.getInt("high_score", 0);
    }

    // Sauvegarder le meilleur score (stub)
    private void saveHighScore() {
        /* SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putInt("high_score", highScore);
            editor.apply(); // ou editor.commit()*/
    }

    public void addNewNumbers() {
        List<int[]> emptySpaces = new ArrayList<>(); // Utilisation de List

        for (int x = 0; x < 4; x++) {
            for (int y = 0; y < 4; y++) {
                if (gameBoard[x][y] == 0) {
                    emptySpaces.add(new int[]{x, y});
                }
            }
        }

        if (!emptySpaces.isEmpty()) {
            int choice = random.nextInt(emptySpaces.size());
            int[] coordinates = emptySpaces.get(choice);
            int x = coordinates[0];
            int y = coordinates[1];
            int newNumber;
            int randomNumber = random.nextInt(100);

            // Switch comme dans le snippet (même si if/else if était correct aussi)
            if (randomNumber < 85) {
                newNumber = 2;
            } else if (randomNumber < 95) {
                newNumber = 4;
            } else {
                newNumber = 8;
            }

            gameBoard[x][y] = newNumber;
        }
    }

    // Les méthodes pushX appellent updateHighScore maintenant

    public void pushUp() {
        Log.d("Game", "Pushing Up");
        boolean[] alreadyCombined = new boolean[4];

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4];
            for (int x = 1; x < 4; x++) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    while (currentX > 0 && gameBoard[currentX - 1][y] == 0) {
                        currentX--;
                    }

                    if (currentX == 0) {
                        gameBoard[0][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX - 1][y] != value) {
                        gameBoard[currentX][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX - 1]) {
                        gameBoard[currentX - 1][y] *= 2;
                        score += gameBoard[currentX - 1][y];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentX - 1] = true;
                        updateHighScore(); // Appel après fusion
                    } else {
                        gameBoard[currentX][y] = value;
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }

    public void pushDown() {
        Log.d("Game", "Pushing Down");
        boolean[] alreadyCombined = new boolean[4];

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4];
            for (int x = 2; x >= 0; x--) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    while (currentX < 3 && gameBoard[currentX + 1][y] == 0) {
                        currentX++;
                    }

                    if (currentX == 3) {
                        gameBoard[3][y] = value;
                        if(currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX + 1][y] != value) {
                        gameBoard[currentX][y] = value;
                        if(currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX + 1]) {
                        gameBoard[currentX + 1][y] *= 2;
                        score += gameBoard[currentX + 1][y];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentX + 1] = true;
                        updateHighScore(); // Appel après fusion

                    }
                    else {
                        gameBoard[currentX][y] = value;
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }

    public void pushLeft() {
        Log.d("Game", "Pushing Left");
        boolean[] alreadyCombined = new boolean[4];

        for (int x = 0; x < 4; x++) {
            alreadyCombined = new boolean[4];
            for (int y = 1; y < 4; y++) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentY = y;

                    while (currentY > 0 && gameBoard[x][currentY - 1] == 0) {
                        currentY--;
                    }

                    if (currentY == 0) {
                        gameBoard[x][0] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[x][currentY - 1] != value) {
                        gameBoard[x][currentY] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentY - 1]) {
                        gameBoard[x][currentY - 1] *= 2;
                        score += gameBoard[x][currentY - 1];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentY - 1] = true;
                        updateHighScore(); // Appel après fusion
                    } else {
                        gameBoard[x][currentY] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    }
                }
            }
        }
    }

    public void pushRight() {
        Log.d("Game", "Pushing Right");
        boolean[] alreadyCombined = new boolean[4];

        for (int x = 0; x < 4; x++) {
            alreadyCombined = new boolean[4];
            for (int y = 2; y >= 0; y--) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentY = y;

                    while (currentY < 3 && gameBoard[x][currentY + 1] == 0) {
                        currentY++;
                    }

                    if (currentY == 3) {
                        gameBoard[x][3] = value;
                        if (currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[x][currentY + 1] != value) {
                        gameBoard[x][currentY] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentY + 1]) {
                        gameBoard[x][currentY + 1] *= 2;
                        score += gameBoard[x][currentY + 1];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentY + 1] = true;
                        updateHighScore(); // Appel après fusion
                    }
                    else{
                        gameBoard[x][currentY] = value;
                        if (currentY != y)
                            gameBoard[x][y] = 0;
                    }
                }
            }
        }
    }

    // Nouvelle méthode pour mettre à jour le high score
    private void updateHighScore() {
        if (score > highScore) {
            highScore = score;
            saveHighScore(); // Sauvegarder (même si le stub est vide)
        }
    }

    // La méthode printArray reste définie comme dans l'extrait précédent
    public void printArray() {
        for (int[] row : gameBoard) {
            String rowString = String.format("%6d%6d%6d%6d%n", row[0], row[1], row[2], row[3]);
            Log.d("Game", rowString);
        }
        Log.d("Game", "\n");
    }
}