package legion.muyue.best2048;

import android.content.Context;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;

public class OnSwipeTouchListener implements View.OnTouchListener {

    private final GestureDetector gestureDetector;
    private final SwipeListener listener; // Ajout de l'interface listener

    // Interface pour les événements de swipe
    public interface SwipeListener {
        void onSwipeTop();
        void onSwipeBottom();
        void onSwipeLeft();
        void onSwipeRight();
    }

    // Constructeur modifié pour accepter le listener
    public OnSwipeTouchListener(Context ctx, SwipeListener listener) {
        gestureDetector = new GestureDetector(ctx, new GestureListener());
        this.listener = listener;
    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        return gestureDetector.onTouchEvent(event);
    }

    private final class GestureListener extends GestureDetector.SimpleOnGestureListener {

        private static final int SWIPE_THRESHOLD = 100;
        private static final int SWIPE_VELOCITY_THRESHOLD = 100;

        @Override
        public boolean onDown(MotionEvent e) {
            return true;
        }

        @Override
        public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
            boolean result = false;
            try {
                float diffY = e2.getY() - e1.getY();
                float diffX = e2.getX() - e1.getX();
                if (Math.abs(diffX) > Math.abs(diffY)) {
                    if (Math.abs(diffX) > SWIPE_THRESHOLD && Math.abs(velocityX) > SWIPE_VELOCITY_THRESHOLD) {
                        if (diffX > 0) {
                            listener.onSwipeRight(); // Appel via listener
                        } else {
                            listener.onSwipeLeft(); // Appel via listener
                        }
                        result = true;
                    }
                } else if (Math.abs(diffY) > SWIPE_THRESHOLD && Math.abs(velocityY) > SWIPE_VELOCITY_THRESHOLD) {
                    if (diffY > 0) {
                        listener.onSwipeBottom(); // Appel via listener
                    } else {
                        listener.onSwipeTop(); // Appel via listener
                    }
                    result = true;
                }
            } catch (Exception exception) {
                exception.printStackTrace();
            }
            return result;
        }
    }
    // Les méthodes locales vides ne sont plus nécessaires car on utilise l'interface
}